function(cleanup varname)
    string(REPLACE "-" "_" out ${${varname}})
    SET(${varname} ${out} PARENT_SCOPE)
endfunction()

if (DEFINED ENV{CI_COMMIT_REF_NAME})
    # this is gitlab CI
    if (DEFINED ENV{CI_COMMIT_TAG})
        # both refname and tag are defined - this is a tagged build
        SET(ASAPO_VERSION $ENV{CI_COMMIT_TAG})
        SET(PYTHON_ASAPO_VERSION ${ASAPO_VERSION})
        SET(ASAPO_VERSION_COMMIT "")
        SET(ASAPO_VERSION_DOCKER_TAG $ENV{CI_COMMIT_TAG})
        SET(ASAPO_VERSION_DOCKER_SUFFIX "")
    else ()
        # tag is not defined, this is the developer build
        SET(ASAPO_VERSION $ENV{CI_COMMIT_REF_NAME})
        string(REPLACE "_" "-" ASAPO_VERSION ${ASAPO_VERSION})
        SET(ASAPO_VERSION 100.0.${ASAPO_VERSION})
        SET(PYTHON_ASAPO_VERSION 100.0.dev1)
        SET(ASAPO_VERSION_COMMIT ", build $ENV{CI_COMMIT_SHORT_SHA}")
        SET(ASAPO_VERSION_DOCKER_TAG $ENV{CI_COMMIT_SHORT_SHA})
        SET(ASAPO_VERSION_DOCKER_SUFFIX "-dev")
    endif ()

    SET(ASAPO_VERSION_DOCKER_REPOSITORY $ENV{CI_REGISTRY_IMAGE})
    SET(ASAPO_VERSION_DOCKER_REPOSITORY_USERNAME $ENV{CI_REGISTRY_IMAGE_USERNAME})
    SET(ASAPO_VERSION_DOCKER_REPOSITORY_PASSWORD $ENV{CI_REGISTRY_IMAGE_PASSWORD})

    message("Asapo Version: " ${ASAPO_VERSION})
    message("Python Asapo Version: " ${PYTHON_ASAPO_VERSION})
    message("Asapo commit: " ${ASAPO_VERSION_COMMIT})
    return ()
endif ()

execute_process(COMMAND git describe --tags --abbrev=0
        OUTPUT_VARIABLE ASAPO_TAG
        WORKING_DIRECTORY ..)
string(STRIP ${ASAPO_TAG} ASAPO_TAG)

execute_process(COMMAND git rev-parse --abbrev-ref HEAD
        OUTPUT_VARIABLE BRANCH
        WORKING_DIRECTORY ..)
string(STRIP ${BRANCH} BRANCH)
cleanup(BRANCH)

execute_process(COMMAND git rev-parse --short=10 HEAD
        OUTPUT_VARIABLE ASAPO_VERSION_COMMIT
        WORKING_DIRECTORY ..)
string(STRIP ${ASAPO_VERSION_COMMIT} ASAPO_VERSION_COMMIT)

if (${BRANCH} STREQUAL "master")
    SET(ASAPO_VERSION ${ASAPO_TAG})
    SET(ASAPO_VERSION_COMMIT "")
    SET(ASAPO_VERSION_DOCKER_SUFFIX "")
    SET(PYTHON_ASAPO_VERSION ${ASAPO_VERSION})
    string(REGEX REPLACE "\\.0([0-9]+)\\."
            ".\\1." ASAPO_WHEEL_VERSION
            ${ASAPO_VERSION})
else ()
    SET(ASAPO_VERSION ${BRANCH})
    SET(ASAPO_VERSION_COMMIT ", build ${ASAPO_VERSION_COMMIT}")
    SET(ASAPO_VERSION_DOCKER_SUFFIX "-dev")
    string(REPLACE "_" "-" ASAPO_VERSION ${ASAPO_VERSION})
    SET(ASAPO_VERSION 100.0.${ASAPO_VERSION})
    if (${BRANCH} STREQUAL "develop")
        SET(PYTHON_ASAPO_VERSION 100.0.dev0)
    else ()
        string(FIND ${BRANCH} feature_ASAPO pos)
        if( ${pos} EQUAL 0)
            string(SUBSTRING ${ASAPO_VERSION} 20 -1 TMP)
            string(REGEX MATCH "^([0-9]+)|.+$" ISSUE_NUM "${TMP}")
            if (ISSUE_NUM STREQUAL "")
                SET(PYTHON_ASAPO_VERSION 100.0.dev1)
            else ()
                SET(PYTHON_ASAPO_VERSION 100.0.dev${ISSUE_NUM})
            endif ()
        else ()
            SET(PYTHON_ASAPO_VERSION 100.0.dev1)
        endif ()
    endif ()
    SET(ASAPO_WHEEL_VERSION ${ASAPO_VERSION})
endif ()

SET(ASAPO_VERSION_DOCKER_REPOSITORY yakser)
SET(ASAPO_VERSION_DOCKER_REPOSITORY_USERNAME "")
SET(ASAPO_VERSION_DOCKER_REPOSITORY_PASSWORD "")
SET(ASAPO_VERSION_DOCKER_TAG ${ASAPO_VERSION})

message("Asapo Version: " ${ASAPO_VERSION})
message("Python Asapo Version: " ${PYTHON_ASAPO_VERSION})
message("Asapo commit: " ${ASAPO_VERSION_COMMIT})
