#include "request_handler_db_persist_stream.h"
#include <asapo/json_parser/json_parser.h>
#include <asapo/database/db_error.h>

#include "../receiver_logger.h"
#include "../receiver_config.h"
#include <sstream>

namespace asapo {

    RequestHandlerDbPersistStream::RequestHandlerDbPersistStream(std::string collection_name_prefix) : RequestHandlerDb(
            std::move(collection_name_prefix)) {
        MaxNumPersistedStreams = (int)GetReceiverConfig()->max_num_persisted_streams;
    }

    Error RequestHandlerDbPersistStream::ProcessRequest(Request* request) const {
        if (auto err = RequestHandlerDb::ProcessRequest(request) ) {
            return err;
        }

        int stream_num;
        auto err = db_client__->GetPersistedStreamsNumber(&stream_num);

        if (err) {
            return DBErrorToReceiverError(std::move(err));
        }

        if (stream_num >= MaxNumPersistedStreams) {
            return ReceiverErrorTemplates::kBadRequest.Generate("too many persisted streams");
        }

        err = db_client__->PersistStream(request->GetStream());

        if (err) {
            return DBErrorToReceiverError(std::move(err));
        }

        return nullptr;
    }


}